@@ -438,6 +438,11 @@ static const struct FREEBLVectorStr vector = {
     Kyber_Decapsulate,
 
     /* End of version 3.027 */
+
+    ED_SignMessage,
+    ED_VerifyMessage,
+    ED_DerivePublicKey,
+    /* End of version 3.028 */
 };
 
 const FREEBLVector*
